package org.netkernel.module.standard.builtin;


import org.netkernel.container.ILogger;
import org.netkernel.container.IMessages;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.module.standard.endpoint.TransparentConfiguredOverlayImpl;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.nkf.impl.NKFContextImpl;
import org.netkernel.layer0.nkf.impl.NKFRequestImpl;
import org.netkernel.layer0.representation.WrappedThrowable;
import org.netkernel.layer0.util.RequestBuilder;
import org.netkernel.layer0.util.RequestBuilder.Arguments;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.ISpace;
import org.netkernel.util.Utils;
import org.w3c.dom.Document;

public class PluggableOverlay extends TransparentConfiguredOverlayImpl
{
	public PluggableOverlay()
	{	declareThreadSafe();
		declareParameters(PluggableOverlayPrototype.sPrototypeParameters);
		declareUnhandledExceptionsExpired(false);
	}
	
	@Override
	protected TransparentConfig createTransparentConfig(INKFRequestContext aContext) throws Exception
	{	IMessages messages=getKernel().getLogger();
	
		RequestBuilder postRequestBuilder=null;
		if (aContext.exists("param:postProcess"))
		{	Document d=aContext.source("param:postProcess",Document.class);
			postRequestBuilder=new RequestBuilder(d,messages);
		}
		RequestBuilder preRequestBuilder=null;
		if (aContext.exists("param:preProcess"))
		{	Document d=aContext.source("param:preProcess",Document.class);
			preRequestBuilder=new RequestBuilder(d,messages);
		}
		RequestBuilder exRequestBuilder=null;
		if (aContext.exists("param:exceptionProcess"))
		{	Document d=aContext.source("param:exceptionProcess",Document.class);
			exRequestBuilder=new RequestBuilder(d,messages);
		}
		
		return new PluggableConfig(preRequestBuilder,postRequestBuilder,exRequestBuilder,getDelegateSpace(),getDelegateMeta(aContext),aContext,this);
	}

	@Override
	protected void destroyConfig(IConfig config) throws Exception
	{
	}

	private static class PluggableConfig extends TransparentConfig
	{
		private RequestBuilder mPreProcess;
		private RequestBuilder mPostProcess;
		private RequestBuilder mExProcess;
		
		public PluggableConfig(RequestBuilder aPreProcess, RequestBuilder aPostProcess, RequestBuilder aExProcess, ISpace aDelegateSpace, ISpaceMeta aDelegateMeta, INKFRequestContext aContext, IEndpoint aEndpoint) throws Exception
		{
			super(aDelegateSpace,aDelegateMeta,aContext,aEndpoint);
			
			mPreProcess=aPreProcess;
			mPostProcess=aPostProcess;
			mExProcess=aExProcess;
		}
		
		public void onRequest(String elementId, INKFRequestContext aContext) throws Exception
		{
			INKFRequestReadOnly reqOut=aContext.getThisRequest();
			INKFRequest reqIn;
			
			if (mPreProcess!=null)
			{	Arguments args = new Arguments();
				args.addArgumentByValue("arg:request",reqOut);
				INKFRequest preReq=mPreProcess.buildRequest(aContext, args,null);
				preReq.injectRequestScope(getDelegateSpace());
				INKFResponseReadOnly resp=aContext.issueRequestForResponse(preReq);
				Object preRepresentation = resp.getRepresentation();
				if (preRepresentation instanceof NKFRequestImpl)
				{	
					NKFRequestImpl req=(NKFRequestImpl)preRepresentation;
					req.setContext((NKFContextImpl)aContext); //ensure parent will be inherited correctly
					reqIn=req;
				}
				else
				{	ILogger logger=aContext.getKernelContext().getKernel().getLogger();
					throw Utils.createFormattedException("EX_STD_PLUGGABLE_PRE_RESP", "MSG_STD_PLUGGABLE_PRE_RESP", logger, preRepresentation.getClass().getName());
				}
			}
			else
			{	reqIn=reqOut.getIssuableClone();
			}
			
			reqIn.injectRequestScope(getDelegateSpace());
			try
			{	INKFResponseReadOnly resp=aContext.issueRequestForResponse(reqIn);

				if (mPostProcess!=null)
				{	Arguments args = new Arguments();
					args.addArgumentFromResponse("arg:response",resp);
					args.addArgumentByValue("arg:request",reqOut);
					INKFRequest postReq=mPostProcess.buildRequest(aContext, args, null);
					postReq.injectRequestScope(getDelegateSpace());
					postReq.setRepresentationClass(reqOut.getRepresentationClass());
					resp=aContext.issueRequestForResponse(postReq);
				}
				aContext.createResponseFrom(resp);
			}
			catch (NKFException e)
			{
				if (mExProcess!=null)
				{	Arguments args = new Arguments();
					args.addArgumentByValue("arg:exception",new WrappedThrowable(e));
					args.addArgumentByValue("arg:request",reqOut);
					INKFRequest exReq=mExProcess.buildRequest(aContext, args, null);
					exReq.injectRequestScope(getDelegateSpace());
					exReq.setRepresentationClass(reqOut.getRepresentationClass());
					INKFResponseReadOnly resp=aContext.issueRequestForResponse(exReq);
					aContext.createResponseFrom(resp);
				}
				else
				{	throw e;
				}
			}
		}
	}
}