package org.netkernel.module.standard.endpoint;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.meta.IPhysicalEndpointFieldMeta;
import org.netkernel.layer0.meta.ISpaceMeta;
import org.netkernel.layer0.meta.impl.PhysicalEndpointFieldMetaImpl;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.util.Utils;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.RequestResponseFieldsImpl;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.urii.ISpace;

public abstract class OverlayImpl extends StandardEndpointImpl implements IStandardOverlay
{
	private ISpace mDelegateSpace;
	private INKFResponseReadOnly<ISpaceMeta> mDelegateMetaResponse;
	
	public void onCommissionEndpoint(IKernel aKernel, ISpace aSpace) throws Exception
	{	super.onCommissionEndpoint(aKernel, aSpace);
		mDelegateSpace=(ISpace)getParameter(PARAM_SPACE);
	}
	
	public IRequestResponseFields getState()
	{	RequestResponseFieldsImpl state=new RequestResponseFieldsImpl(super.getState());
		state.put(STATE_DELEGATE_SPACE, mDelegateSpace);
		return state;
	}
	
	private static IPhysicalEndpointFieldMeta[] sFieldMetas = new IPhysicalEndpointFieldMeta[1];
	static
	{	sFieldMetas[0] = new PhysicalEndpointFieldMetaImpl("reset","reset the statistics",Void.class,true);
	}
	
	public IPhysicalEndpointFieldMeta[] getStateMeta()
	{	return sFieldMetas;
	}
	
	public void setState(String aKey, Object aValue) throws IllegalArgumentException
	{	if (aKey.equals("reset"))
		{	System.out.println("reset "+this);
		}
		else
		{	throw new IllegalArgumentException();
		}
	}	
	
	void doPostCommission(INKFRequestContext aContext) throws Exception
	{	mDelegateSpace.postCommissionSpace();
		postCommission(aContext);
	}	

	public void preDecommission(INKFRequestContext aContext) throws Exception
	{	mDelegateSpace.preDecommissionSpace();
		super.preDecommission(aContext);
	}
	
	public void onDecommissionEndpoint() throws Exception
	{	mDelegateSpace.onDecommissionSpace();
		super.onDecommissionEndpoint();
	}

	public ISpace getDelegateSpace()
	{	return mDelegateSpace;
	}
	
	public void setDelegateSpace(ISpace aSpace) throws NKFException
	{	setParameterRaw(PARAM_SPACE, aSpace, null);
	}
	
	protected ISpaceMeta getDelegateMeta(INKFRequestContext aContext) throws NKFException
	{	INKFResponseReadOnly<ISpaceMeta> metaResp=mDelegateMetaResponse;
		if (metaResp==null || metaResp.isExpired())
		{	ISpace delegateSpace = getDelegateSpace();
			INKFRequest req = aContext.createRequest("");
			req.setVerb(INKFRequestReadOnly.VERB_META);
			IRequestScopeLevel rctx = RequestScopeLevelImpl.createOrphanedRootScopeLevel(delegateSpace,aContext.getKernelContext().getThisKernelRequest().getRequestScope());
			req.setRequestScope(rctx);
			req.setRepresentationClass(ISpaceMeta.class);
			metaResp=aContext.issueRequestForResponse(req);
			if (!metaResp.isExpired())
			{	mDelegateMetaResponse=metaResp;
			}
		}
		else
		{	aContext.getKernelContext().declareDependency(metaResp);
		}
		return metaResp.getRepresentation();
	}
	
	public String toString()
	{	return Utils.shortClassName(this.getClass());
	}

	public ISpace[] getSpaces(INKFRequestContext context) throws Exception
	{	return new ISpace[] {mDelegateSpace};
	}
	
}
